@@ -1,11 +1,11 @@
 ### -*- coding: utf-8 -*- #############################################
 # Developed by Maksym Polshcha (maxp@sterch.net)
-# All right reserved, 2012, 2013
+# All right reserved, 2012, 2013, 2014
 #######################################################################
 
 # Make it a Python package
 from courts import is_plaintiff, is_defendant, is_attorney, extract_description, extract_date, extract_money, is_john_doe, is_valid_attorney, SequenceState
-from opener import createOpener, readpage, Client, BaseCaptchaAwareClient, clone_client
+from opener import createOpener, readpage, Client, clone_client
 from output import start_chunked_stdout, stop_chunked_stdout
 from synclist import SyncList, DuplicateValueError
 from text import replace_html_entities, striptags, normalize, tofilename, parse_fullname, parse_fulladdress
